@@ -9,11 +9,13 @@ import sys
 import tempfile
 import textwrap
 from contextlib import contextmanager
+from pprint import pprint
 
 import netifaces
+import requests
 
 from fig.cli import main
-
+from version import __version__
 
 ROOT = os.path.dirname(os.path.abspath(__file__))
 join = functools.partial(os.path.join, ROOT)
@@ -29,26 +31,30 @@ BRANCHES = [
     'zippy',
 ]
 
+MIGRATIONS = ['zamboni', 'solitude']
+
+SERVICE_CHECKS = {
+    'solitude': 'http://mp.dev/solitude/services/status/',
+    'webpay': 'http://mp.dev/mozpay/services/monitor',
+    'zamboni': 'http://mp.dev/services/monitor.json'
+}
+
 
 # Command functions:
 
 def check_git_config(args, parser):
     for branch in BRANCHES:
         branch_dir = join(locations()['tree'], branch)
-        cur_dir = os.getcwd()
-        if os.path.isdir(branch_dir):
-            try:
-                os.chdir(branch_dir)
-                print "[{0}]".format(branch)
-                indent("[remotes]")
-                indent(subprocess.check_output(['git', 'remote', '-v']), 2)
-                indent("[Master branch origin]")
-                origin = subprocess.check_output(['git', 'config', '--get',
-                                                  'branch.master.remote'])
-                indent(origin, 2)
-                print
-            finally:
-                os.chdir(cur_dir)
+        with pushd(branch_dir):
+            os.chdir(branch_dir)
+            print "[{0}]".format(branch)
+            indent("[remotes]")
+            indent(subprocess.check_output(['git', 'remote', '-v']), 2)
+            indent("[Master branch origin]")
+            origin = subprocess.check_output(['git', 'config', '--get',
+                                              'branch.master.remote'])
+            indent(origin, 2)
+            print
 
 
 def checkout(args, parser, gh_username=None):
@@ -100,13 +106,6 @@ def whoami(args=None, parser=None, quiet=False):
     return user
 
 
-def shell(args, parser):
-    image = get_image(args, parser)
-    image_name = image['name']
-    return subprocess.call(['docker', 'run', '-a', 'stdin', '-a', 'stdout',
-                            '-i', '-t', image_name + '_img', '/bin/bash'])
-
-
 def locations():
     return {
         # Where the checked out projects live.
@@ -152,9 +151,15 @@ def update_config(args, parser):
 
 def up(args, parser):
     update_config(args, parser)
-    main.setup_logging()
-    cmd = main.TopLevelCommand()
-    cmd.dispatch(['up', '-d'], None)
+    fig_command('up', '-d', '--no-recreate')
+
+
+def bash(args, parser):
+    project = get_project(args.project)
+    cmd = ('docker exec -t -i {0} /bin/bash'
+           .format(get_fig_container(project).id))
+    os.system(cmd)
+    return
 
 
 def check(args, parser):
@@ -185,6 +190,52 @@ def check(args, parser):
             print ('Directory {0} does not exist, run checkout.'
                    .format(branch_dir))
 
+    if args.services:
+        for service, url in SERVICE_CHECKS.items():
+            try:
+                res = requests.get(url, timeout=5)
+            except (requests.exceptions.Timeout,
+                    requests.exceptions.ConnectionError):
+                # nginx isn't even up.
+                print 'Error on: {0}, is it running?'.format(service)
+                continue
+
+            if res.status_code == 502:
+                # nginx proxy errors.
+                print 'Service not up: {0} (proxy error)'.format(service)
+
+            if res.status_code == 500:
+                print 'Status failed on: {0}.'.format(service)
+                print
+                pprint(res.json())
+                print
+
+
+def update(args, parser):
+    git, migration = args.git, args.migrations
+    if not git and not migration:
+        # If the user didn't pass a flag, run both.
+        git, migration = True, True
+
+    if git:
+        for branch in BRANCHES:
+            branch_dir = join(locations()['tree'], branch)
+            with pushd(branch_dir):
+                try:
+                    print 'Updating git for: {0}'.format(branch)
+                    indent(subprocess.check_output(['git', 'pull', '-q']), 2)
+                except subprocess.CalledProcessError:
+                    print
+                    print 'Failed to update: {0}'.format(branch_dir)
+                    print
+                    raise
+
+    if migration:
+        for migration in MIGRATIONS:
+            print 'Running migration for: {0}'.format(migration)
+            fig_command('run', '--rm', migration,
+                        'schematic', 'migrations')
+
 
 def bind(args, parser):
     if args.interfaces:
@@ -253,6 +304,43 @@ def bind(args, parser):
 
 # Helper functions:
 
+def get_project(project):
+    cur = os.getcwd()
+
+    def walk(directory):
+        if 'Dockerfile' in os.listdir(directory):
+            return os.path.basename(directory)
+
+        new = os.path.dirname(directory)
+        if new == directory:
+            raise ValueError('No project found.')
+        return walk(new)
+
+    project = project or walk(cur)
+    if project not in BRANCHES:
+        raise ValueError('Project {0} not in BRANCHES'.format(project))
+
+    return project
+
+
+def get_fig_container(project):
+    cmd = main.Command()
+    proj = cmd.get_project(FIG_PATH)
+    containers = proj.containers(service_names=[project])
+    if not containers:
+        raise ValueError('No containers found for: {0}'.format(project))
+    return containers[0]
+
+
+def fig_command(*args):
+    cmd = main.TopLevelCommand()
+    try:
+        cmd.dispatch(args, None)
+    except SystemExit as exit:
+        if exit.code != 0:
+            raise
+
+
 def get_config_value(section, key, default=None):
     config = configparser.ConfigParser()
     config.read(CONFIG_PATH)
@@ -400,9 +488,31 @@ def create_parser():
     )
     parser_root.set_defaults(func=root)
 
+    parser_bash = subparsers.add_parser(
+        'bash', help='Run a bash shell on a running container.'
+    )
+    parser_bash.add_argument(
+        '--project',
+        help='Project name, if not given will be calculated.',
+        action='store')
+    parser_bash.set_defaults(func=bash)
+
+    parser_update = subparsers.add_parser(
+        'update', help='Runs git pull on each repo and any migrations.'
+    )
+    parser_update.add_argument(
+        '--git', help='Runs git pull', action='store_true')
+    parser_update.add_argument(
+        '--migrations', help='Runs migrations', action='store_true')
+    parser_update.set_defaults(func=update)
+
     parser_check = subparsers.add_parser(
         'check', help='Basic health checks of the system.'
     )
+    parser_check.add_argument(
+        '--services', help='Checks the status page of each service.',
+        action='store_true'
+    )
     parser_check.set_defaults(func=check)
 
     parser_up = subparsers.add_parser(
@@ -411,21 +521,13 @@ def create_parser():
     )
     parser_up.set_defaults(func=up)
 
-    parser_shell = subparsers.add_parser(
-        'shell', help='Run image, and drop into a shell on it.'
-    )
-    parser_shell.add_argument(
-        'name', help='The name of the image to run a shell on',
-        metavar="IMAGE_NAME",
-    )
-    parser_shell.set_defaults(func=shell)
-    parser_shell = subparsers.add_parser(
+    parser_checkgitconfig = subparsers.add_parser(
         'chkgitconfig', help='Print out the git config for mkt branches'
     )
-    parser_shell.set_defaults(func=check_git_config)
+    parser_checkgitconfig.set_defaults(func=check_git_config)
 
     parser_whoami = subparsers.add_parser(
-        'whoami', help='Check your github credentials'
+        'whoami', help='Check or store your github credentials'
     )
     parser_whoami.add_argument(
         'github_username', help='Your github username e.g. "jrrtolkien"',
@@ -472,5 +574,7 @@ def create_parser():
         action='store_true')
     parser_bind.set_defaults(func=bind)
 
+    parser.add_argument('--version', action='version', version=__version__)
+    # Setup the logging for fig.
+    main.setup_logging()
     return parser
-
